@@ -13,6 +13,7 @@
 #
 # Copyright Buildbot Team Members
 
+import base64
 import json
 import os
 import random
@@ -62,7 +63,7 @@ class SourceStampExtractor:
         self.branch = branch
         exes = which(self.vcexe)
         if not exes:
-            output("Could not find executable '{}'.".format(self.vcexe))
+            output(f"Could not find executable '{self.vcexe}'.")
             sys.exit(1)
         self.exe = exes[0]
 
@@ -77,7 +78,7 @@ class SourceStampExtractor:
         return d
 
     def _didvc(self, res, cmd):
-        (stdout, stderr, code) = res
+        stdout, _, __ = res
         # 'bzr diff' sets rc=1 if there were any differences.
         # cvs does something similar, so don't bother requiring rc=0.
         return stdout
@@ -92,13 +93,13 @@ class SourceStampExtractor:
     def readPatch(self, diff, patchlevel):
         if not diff:
             diff = None
-        self.patch = (patchlevel, bytes2unicode(diff))
+        self.patch = (patchlevel, diff)
 
     def done(self, res):
         if not self.repository:
             self.repository = self.treetop
         # TODO: figure out the branch and project too
-        ss = SourceStamp(self.branch, self.baserev, self.patch,
+        ss = SourceStamp(bytes2unicode(self.branch), self.baserev, self.patch,
                          repository=self.repository)
         return ss
 
@@ -170,7 +171,7 @@ class SVNExtractor(SourceStampExtractor):
         sys.exit(1)
 
     def getPatch(self, res):
-        d = self.dovc(["diff", "-r{}".format(self.baserev)])
+        d = self.dovc(["diff", f"-r{self.baserev}"])
         d.addCallback(self.readPatch, self.patchlevel)
         return d
 
@@ -185,12 +186,12 @@ class BzrExtractor(SourceStampExtractor):
         return d
 
     def get_revision_number(self, out):
-        revno, revid = out.split()
+        _, revid = out.split()
         self.baserev = 'revid:' + revid
         return
 
     def getPatch(self, res):
-        d = self.dovc(["diff", "-r{}..".format(self.baserev)])
+        d = self.dovc(["diff", f"-r{self.baserev}.."])
         d.addCallback(self.readPatch, self.patchlevel)
         return d
 
@@ -206,7 +207,7 @@ class MercurialExtractor(SourceStampExtractor):
             cs = ' '.join(['hg'] + cmd)
             if stderr:
                 stderr = '\n' + stderr.rstrip()
-            raise RuntimeError("{} returned {} {}".format(cs, code, stderr))
+            raise RuntimeError(f"{cs} returned {code} {stderr}")
 
         return stdout
 
@@ -214,11 +215,11 @@ class MercurialExtractor(SourceStampExtractor):
     def getBaseRevision(self):
         upstream = ""
         if self.repository:
-            upstream = "r'{}'".format(self.repository)
+            upstream = f"r'{self.repository}'"
         output = ''
         try:
             output = yield self.dovc(["log", "--template", "{node}\\n", "-r",
-                                      "max(::. - outgoing({}))".format(upstream)])
+                                      f"max(::. - outgoing({upstream}))"])
         except RuntimeError:
             # outgoing() will abort if no default-push/default path is
             # configured
@@ -228,8 +229,7 @@ class MercurialExtractor(SourceStampExtractor):
             output = yield self.dovc(["log", "--template", "{node}\\n", "-r", "p1()"])
         m = re.search(br'^(\w+)', output)
         if not m:
-            raise RuntimeError(
-                "Revision {!r} is not in the right format".format(output))
+            raise RuntimeError(f"Revision {output!r} is not in the right format")
         self.baserev = m.group(0)
 
     def getPatch(self, res):
@@ -259,21 +259,21 @@ class PerforceExtractor(SourceStampExtractor):
         output(b"Could not find change number in output: " + res)
         sys.exit(1)
 
-    def readPatch(self, res, patchlevel):
+    def readPatch(self, diff, patchlevel):
         #
-        # extract the actual patch from "res"
+        # extract the actual patch from "diff"
         #
         if not self.branch:
             output("you must specify a branch")
             sys.exit(1)
         mpatch = ""
         found = False
-        for line in res.split("\n"):
+        for line in diff.split("\n"):
             m = re.search('==== //depot/' + self.branch
                           + r'/([\w/\.\d\-_]+)#(\d+) -', line)
             if m:
-                mpatch += "--- {}#{}\n".format(m.group(1), m.group(2))
-                mpatch += "+++ {}\n".format(m.group(1))
+                mpatch += f"--- {m.group(1)}#{m.group(2)}\n"
+                mpatch += f"+++ {m.group(1)}\n"
                 found = True
             else:
                 mpatch += line
@@ -281,7 +281,7 @@ class PerforceExtractor(SourceStampExtractor):
         if not found:
             output(b"could not parse patch file")
             sys.exit(1)
-        self.patch = (patchlevel, mpatch)
+        self.patch = (patchlevel, unicode2bytes(mpatch))
 
     def getPatch(self, res):
         d = self.dovc(["diff"])
@@ -438,51 +438,44 @@ def getSourceStamp(vctype, treetop, branch=None, repository=None):
     elif vctype == "none":
         return defer.succeed(SourceStamp("", "", (1, ""), ""))
     else:
-        output("unknown vctype '{}'".format(vctype))
+        output(f"unknown vctype '{vctype}'")
         sys.exit(1)
     return cls(treetop, branch, repository).get()
 
 
 def ns(s):
-    return "{}:{},".format(len(s), s)
+    return f"{len(s)}:{s},"
 
 
 def createJobfile(jobid, branch, baserev, patch_level, patch_body, repository,
                   project, who, comment, builderNames, properties):
     # Determine job file version from provided arguments
-    if properties:
+    try:
+        bytes2unicode(patch_body)
         version = 5
-    elif comment:
-        version = 4
-    elif who:
-        version = 3
-    else:
-        version = 2
+    except UnicodeDecodeError:
+        version = 6
+
     job = ""
     job += ns(str(version))
-    if version < 5:
-        job += ns(jobid)
-        job += ns(branch)
-        job += ns(str(baserev))
-        job += ns("{}".format(patch_level))
-        job += ns(patch_body or "")
-        job += ns(repository)
-        job += ns(project)
-        if (version >= 3):
-            job += ns(who)
-        if (version >= 4):
-            job += ns(comment)
-        for bn in builderNames:
-            job += ns(bn)
+    job_dict = {
+        'jobid': jobid,
+        'branch': branch,
+        'baserev': str(baserev),
+        'patch_level': patch_level,
+        'repository': repository,
+        'project': project,
+        'who': who,
+        'comment': comment,
+        'builderNames': builderNames,
+        'properties': properties,
+    }
+    if version > 5:
+        job_dict['patch_body_base64'] = bytes2unicode(base64.b64encode(patch_body))
     else:
-        job += ns(
-            json.dumps({
-                'jobid': jobid, 'branch': branch, 'baserev': str(baserev),
-                'patch_level': patch_level, 'patch_body': patch_body,
-                'repository': repository, 'project': project, 'who': who,
-                'comment': comment, 'builderNames': builderNames,
-                'properties': properties,
-            }))
+        job_dict['patch_body'] = bytes2unicode(patch_body)
+
+    job += ns(json.dumps(job_dict))
     return job
 
 
@@ -500,8 +493,7 @@ def getTopdir(topfile, start=None):
             break                       # we've hit the root
         here = next
         toomany -= 1
-    output("Unable to find topfile '{}' anywhere "
-           "from {} upwards".format(topfile, start))
+    output(f"Unable to find topfile '{topfile}' anywhere from {start} upwards")
     sys.exit(1)
 
 
@@ -521,12 +513,11 @@ class RemoteTryPP(protocol.ProcessProtocol):
     def errReceived(self, data):
         sys.stderr.write(bytes2unicode(data))
 
-    def processEnded(self, status_object):
-        sig = status_object.value.signal
-        rc = status_object.value.exitCode
+    def processEnded(self, reason):
+        sig = reason.value.signal
+        rc = reason.value.exitCode
         if sig is not None or rc != 0:
-            self.d.errback(RuntimeError("remote 'buildbot tryserver' failed"
-                                        ": sig={}, rc={}".format(sig, rc)))
+            self.d.errback(RuntimeError(f"remote 'buildbot tryserver' failed: sig={sig}, rc={rc}"))
             return
         self.d.callback((sig, rc))
 
@@ -567,7 +558,7 @@ class Try(pb.Referenceable):
         # generate a random (unique) string. It would make sense to add a
         # hostname and process ID here, but a) I suspect that would cause
         # windows portability problems, and b) really this is good enough
-        self.bsid = "{}-{}".format(time.time(), random.randint(0, 1000000))
+        self.bsid = f"{time.time()}-{random.randint(0, 1000000)}"
 
         # common options
         branch = self.getopt("branch")
@@ -578,7 +569,7 @@ class Try(pb.Referenceable):
             if difffile == "-":
                 diff = sys.stdin.read()
             else:
-                with open(difffile, "r") as f:
+                with open(difffile, "rb") as f:
                     diff = f.read()
             if not diff:
                 diff = None
@@ -624,96 +615,85 @@ class Try(pb.Referenceable):
     def fakeDeliverJob(self):
         # Display the job to be delivered, but don't perform delivery.
         ss = self.sourcestamp
-        output("Job:\n\tRepository: {}\n\tProject: {}\n\tBranch: {}\n\t"
-               "Revision: {}\n\tBuilders: {}\n{}".format(
-               ss.repository, self.project, ss.branch,
-               ss.revision,
-               self.builderNames,
-               ss.patch[1]))
+        output(f"Job:\n\tRepository: {ss.repository}\n\tProject: {self.project}\n\tBranch: "
+               f"{ss.branch}\n\tRevision: {ss.revision}\n\tBuilders: "
+               f"{self.builderNames}\n{ss.patch[1]}")
         self.buildsetStatus = FakeBuildSetStatus()
         d = defer.Deferred()
         d.callback(True)
         return d
 
-    def deliverJob(self):
-        # returns a Deferred that fires when the job has been delivered
-        if self.connect == "ssh":
-            tryhost = self.getopt("host")
-            tryport = self.getopt("port")
-            tryuser = self.getopt("username")
-            trydir = self.getopt("jobdir")
-            buildbotbin = self.getopt("buildbotbin")
-            ssh_command = self.getopt("ssh")
-            if not ssh_command:
-                ssh_commands = which("ssh")
-                if not ssh_commands:
-                    raise RuntimeError("couldn't find ssh executable, make sure "
-                                       "it is available in the PATH")
+    def deliver_job_ssh(self):
+        tryhost = self.getopt("host")
+        tryport = self.getopt("port")
+        tryuser = self.getopt("username")
+        trydir = self.getopt("jobdir")
+        buildbotbin = self.getopt("buildbotbin")
+        ssh_command = self.getopt("ssh")
+        if not ssh_command:
+            ssh_commands = which("ssh")
+            if not ssh_commands:
+                raise RuntimeError("couldn't find ssh executable, make sure "
+                                   "it is available in the PATH")
 
-                argv = [ssh_commands[0]]
+            argv = [ssh_commands[0]]
+        else:
+            # Split the string on whitespace to allow passing options in
+            # ssh command too, but preserving whitespace inside quotes to
+            # allow using paths with spaces in them which is common under
+            # Windows. And because Windows uses backslashes in paths, we
+            # can't just use shlex.split there as it would interpret them
+            # specially, so do it by hand.
+            if runtime.platformType == 'win32':
+                # Note that regex here matches the arguments, not the
+                # separators, as it's simpler to do it like this. And then we
+                # just need to get all of them together using the slice and
+                # also remove the quotes from those that were quoted.
+                argv = [string.strip(a, '"') for a in
+                        re.split(r'''([^" ]+|"[^"]+")''', ssh_command)[1::2]]
             else:
-                # Split the string on whitespace to allow passing options in
-                # ssh command too, but preserving whitespace inside quotes to
-                # allow using paths with spaces in them which is common under
-                # Windows. And because Windows uses backslashes in paths, we
-                # can't just use shlex.split there as it would interpret them
-                # specially, so do it by hand.
-                if runtime.platformType == 'win32':
-                    # Note that regex here matches the arguments, not the
-                    # separators, as it's simpler to do it like this. And then we
-                    # just need to get all of them together using the slice and
-                    # also remove the quotes from those that were quoted.
-                    argv = [string.strip(a, '"') for a in
-                            re.split(r'''([^" ]+|"[^"]+")''', ssh_command)[1::2]]
-                else:
-                    # Do use standard tokenization logic under POSIX.
-                    argv = shlex.split(ssh_command)
+                # Do use standard tokenization logic under POSIX.
+                argv = shlex.split(ssh_command)
 
-            if tryuser:
-                argv += ["-l", tryuser]
+        if tryuser:
+            argv += ["-l", tryuser]
 
-            if tryport:
-                argv += ["-p", tryport]
+        if tryport:
+            argv += ["-p", tryport]
 
-            argv += [tryhost, buildbotbin, "tryserver", "--jobdir", trydir]
-            pp = RemoteTryPP(self.jobfile)
-            reactor.spawnProcess(pp, argv[0], argv, os.environ)
-            d = pp.d
-            return d
-        if self.connect == "pb":
-            user = self.getopt("username")
-            passwd = self.getopt("passwd")
-            master = self.getopt("master")
-            tryhost, tryport = master.split(":")
-            tryport = int(tryport)
-            f = pb.PBClientFactory()
-            d = f.login(credentials.UsernamePassword(unicode2bytes(user), unicode2bytes(passwd)))
-            reactor.connectTCP(tryhost, tryport, f)
-            d.addCallback(self._deliverJob_pb)
-            return d
-        raise RuntimeError("unknown connecttype '{}', "
-                           "should be 'ssh' or 'pb'".format(self.connect))
+        argv += [tryhost, buildbotbin, "tryserver", "--jobdir", trydir]
+        pp = RemoteTryPP(self.jobfile)
+        reactor.spawnProcess(pp, argv[0], argv, os.environ)
+        d = pp.d
+        return d
+
+    @defer.inlineCallbacks
+    def deliver_job_pb(self):
+        user = self.getopt("username")
+        passwd = self.getopt("passwd")
+        master = self.getopt("master")
+        tryhost, tryport = master.split(":")
+        tryport = int(tryport)
+        f = pb.PBClientFactory()
+        d = f.login(credentials.UsernamePassword(unicode2bytes(user), unicode2bytes(passwd)))
+        reactor.connectTCP(tryhost, tryport, f)
+        remote = yield d
 
-    def _deliverJob_pb(self, remote):
         ss = self.sourcestamp
         output("Delivering job; comment=", self.comment)
 
-        d = remote.callRemote("try",
-                              ss.branch,
-                              ss.revision,
-                              ss.patch,
-                              ss.repository,
-                              self.project,
-                              self.builderNames,
-                              self.who,
-                              self.comment,
-                              self.config.get('properties', {}))
-        d.addCallback(self._deliverJob_pb2)
-        return d
+        self.buildsetStatus = \
+            yield remote.callRemote("try", ss.branch, ss.revision, ss.patch, ss.repository,
+                                    self.project, self.builderNames, self.who, self.comment,
+                                    self.config.get('properties', {}))
 
-    def _deliverJob_pb2(self, status):
-        self.buildsetStatus = status
-        return status
+    def deliverJob(self):
+        # returns a Deferred that fires when the job has been delivered
+        if self.connect == "ssh":
+            return self.deliver_job_ssh()
+        if self.connect == "pb":
+            return self.deliver_job_pb()
+        raise RuntimeError(f"unknown connecttype '{self.connect}', should be 'ssh' or 'pb'")
 
     def getStatus(self):
         # returns a Deferred that fires when the builds have finished, and
@@ -725,21 +705,21 @@ class Try(pb.Referenceable):
             output("waiting for builds with ssh is not supported")
         else:
             self.running = defer.Deferred()
-            assert self.buildsetStatus
-            self._getStatus_1()
+            if not self.buildsetStatus:
+                output("try scheduler on the master does not have the builder configured")
+                return None
+
+            self._getStatus_1()  # note that we don't wait for the returned Deferred
             if bool(self.config.get("dryrun")):
                 self.statusDone()
             return self.running
         return None
 
-    def _getStatus_1(self, res=None):
-        if res:
-            self.buildsetStatus = res
+    @defer.inlineCallbacks
+    def _getStatus_1(self):
         # gather the set of BuildRequests
-        d = self.buildsetStatus.callRemote("getBuildRequests")
-        d.addCallback(self._getStatus_2)
+        brs = yield self.buildsetStatus.callRemote("getBuildRequests")
 
-    def _getStatus_2(self, brs):
         self.builderNames = []
         self.buildRequests = {}
 
@@ -799,6 +779,7 @@ class Try(pb.Referenceable):
     def remote_buildETAUpdate(self, buildername, build, eta):
         self.ETA[buildername] = now() + eta
 
+    @defer.inlineCallbacks
     def _build_finished(self, bs, builderName):
         # we need to collect status from the newly-finished build. We don't
         # remove the build from self.outstanding until we've collected
@@ -806,24 +787,13 @@ class Try(pb.Referenceable):
         self.builds[builderName] = None
         self.ETA[builderName] = None
         self.currentStep[builderName] = "finished"
-        d = bs.callRemote("getResults")
-        d.addCallback(self._build_finished_2, bs, builderName)
-        return d
 
-    def _build_finished_2(self, results, bs, builderName):
-        self.results[builderName][0] = results
-        d = bs.callRemote("getText")
-        d.addCallback(self._build_finished_3, builderName)
-        return d
-
-    def _build_finished_3(self, text, builderName):
-        self.results[builderName][1] = text
+        self.results[builderName][0] = yield bs.callRemote("getResults")
+        self.results[builderName][1] = yield bs.callRemote("getText")
 
         self.outstanding.remove(builderName)
         if not self.outstanding:
-            # all done
-            return self.statusDone()
-        return None
+            self.statusDone()
 
     def printStatus(self):
         try:
@@ -834,14 +804,14 @@ class Try(pb.Referenceable):
                     code, text = self.results[n]
                     t = Results[code]
                     if text:
-                        t += " ({})".format(" ".join(text))
+                        t += f' ({" ".join(text)})'
                 elif self.builds[n]:
                     t = self.currentStep[n] or "building"
                     if self.ETA[n]:
-                        t += " [ETA {}s]".format(self.ETA[n] - now())
+                        t += f" [ETA {self.ETA[n] - now()}s]"
                 else:
                     t = "no build"
-                self.announce("{}: {}".format(n, t))
+                self.announce(f"{n}: {t}")
             self.announce("")
         except Exception:
             log.err(None, "printing status")
@@ -856,9 +826,9 @@ class Try(pb.Referenceable):
         happy = True
         for n in names:
             code, text = self.results[n]
-            t = "{}: {}".format(n, Results[code])
+            t = f"{n}: {Results[code]}"
             if text:
-                t += " ({})".format(" ".join(text))
+                t += f' ({" ".join(text)})'
             output(t)
             if code != SUCCESS:
                 happy = False
@@ -869,6 +839,7 @@ class Try(pb.Referenceable):
             self.exitcode = 1
         self.running.callback(self.exitcode)
 
+    @defer.inlineCallbacks
     def getAvailableBuilderNames(self):
         # This logs into the master using the PB protocol to
         # get the names of the configured builders that can
@@ -882,52 +853,55 @@ class Try(pb.Referenceable):
             f = pb.PBClientFactory()
             d = f.login(credentials.UsernamePassword(unicode2bytes(user), unicode2bytes(passwd)))
             reactor.connectTCP(tryhost, tryport, f)
-            d.addCallback(self._getBuilderNames)
-            return d
+            remote = yield d
+            buildernames = yield remote.callRemote("getAvailableBuilderNames")
+
+            output("The following builders are available for the try scheduler: ")
+            for buildername in buildernames:
+                output(buildername)
+
+            yield remote.broker.transport.loseConnection()
+            return
         if self.connect == "ssh":
             output("Cannot get available builders over ssh.")
             sys.exit(1)
-        raise RuntimeError(
-            "unknown connecttype '{}', should be 'pb'".format(self.connect))
-
-    def _getBuilderNames(self, remote):
-        d = remote.callRemote("getAvailableBuilderNames")
-        d.addCallback(self._getBuilderNames2)
-        d.addCallback(lambda _: remote.broker.transport.loseConnection())
-        return d
-
-    def _getBuilderNames2(self, buildernames):
-        output("The following builders are available for the try scheduler: ")
-        for buildername in buildernames:
-            output(buildername)
+        raise RuntimeError(f"unknown connecttype '{self.connect}', should be 'pb'")
 
     def announce(self, message):
         if not self.quiet:
             output(message)
 
-    def run(self, _inTests=False):
-        # we can't do spawnProcess until we're inside reactor.run(), so get
-        # funky
-        output("using '{}' connect method".format(self.connect))
+    @defer.inlineCallbacks
+    def run_impl(self):
+        output(f"using '{self.connect}' connect method")
         self.exitcode = 0
-        d = fireEventually(None)
-        if bool(self.config.get("get-builder-names")):
-            d.addCallback(lambda res: self.getAvailableBuilderNames())
-        else:
-            d.addCallback(lambda res: self.createJob())
-            d.addCallback(lambda res: self.announce("job created"))
-            deliver = self.deliverJob
-            if bool(self.config.get("dryrun")):
-                deliver = self.fakeDeliverJob
-            d.addCallback(lambda res: deliver())
-            d.addCallback(lambda res: self.announce("job has been delivered"))
-            d.addCallback(lambda res: self.getStatus())
-        d.addErrback(self.trapSystemExit)
-        d.addErrback(log.err)
-        if not bool(self.config.get("dryrun")):
-            d.addCallback(self.cleanup)
-        if _inTests:
-            return d
+
+        # we can't do spawnProcess until we're inside reactor.run(), so force asynchronous execution
+        yield fireEventually(None)
+
+        try:
+            if bool(self.config.get("get-builder-names")):
+                yield self.getAvailableBuilderNames()
+            else:
+                yield self.createJob()
+                yield self.announce("job created")
+                if bool(self.config.get("dryrun")):
+                    yield self.fakeDeliverJob()
+                else:
+                    yield self.deliverJob()
+                yield self.announce("job has been delivered")
+                yield self.getStatus()
+
+            if not bool(self.config.get("dryrun")):
+                yield self.cleanup()
+        except SystemExit as e:
+            self.exitcode = e.code
+        except Exception as e:
+            log.err(e)
+            raise
+
+    def run(self):
+        d = self.run_impl()
         d.addCallback(lambda res: reactor.stop())
 
         reactor.run()
