@@ -63,6 +63,7 @@ class RemoteCommand(base.RemoteCommandImpl):
         self.ignore_updates = ignore_updates
         self.decodeRC = decodeRC
         self.conn = None
+        self._is_conn_test_fake = False
         self.worker = None
         self.step = None
         self.builder_name = None
@@ -75,7 +76,7 @@ class RemoteCommand(base.RemoteCommandImpl):
         self.loglock = defer.DeferredLock()
 
     def __repr__(self):
-        return "<RemoteCommand '{}' at {}>".format(self.remote_command, id(self))
+        return f"<RemoteCommand '{self.remote_command}' at {id(self)}>"
 
     def run(self, step, conn, builder_name):
         self.active = True
@@ -83,12 +84,15 @@ class RemoteCommand(base.RemoteCommandImpl):
         self.conn = conn
         self.builder_name = builder_name
 
+        # This probably could be solved in a cleaner way.
+        self._is_conn_test_fake = hasattr(self.conn, 'is_fake_test_connection')
+
         # generate a new command id
         cmd_id = RemoteCommand._commandCounter
         RemoteCommand._commandCounter += 1
-        self.commandID = "%d" % cmd_id
+        self.commandID = f"{cmd_id}"
 
-        log.msg("{}: RemoteCommand.run [{}]".format(self, self.commandID))
+        log.msg(f"{self}: RemoteCommand.run [{self.commandID}]")
         self.deferred = defer.Deferred()
 
         d = defer.maybeDeferred(self._start)
@@ -133,55 +137,56 @@ class RemoteCommand(base.RemoteCommandImpl):
 
     @defer.inlineCallbacks
     def _finished(self, failure=None):
+        # Finished may be called concurrently by a message from worker and interruption due to
+        # lost connection.
+        if not self.active:
+            return
         self.active = False
+
         # the rc is send asynchronously and there is a chance it is still in the callback queue
         # when finished is received, we have to workaround in the master because worker might be
         # older
-        timeout = 10
-        while self.rc is None and timeout > 0:
-            yield util.asyncSleep(.1)
-            timeout -= 1
-        # call .remoteComplete. If it raises an exception, or returns the
-        # Failure that we gave it, our self.deferred will be errbacked. If
-        # it does not (either it ate the Failure or there the step finished
-        # normally and it didn't raise a new exception), self.deferred will
-        # be callbacked.
-        d = defer.maybeDeferred(self.remoteComplete, failure)
-        # arrange for the callback to get this RemoteCommand instance
-        # instead of just None
-        d.addCallback(lambda r: self)
-        # this fires the original deferred we returned from .run(),
-        # with self as the result, or a failure
-        d.addBoth(self.deferred.callback)
+        if not self._is_conn_test_fake:
+            timeout = 10
+            while self.rc is None and timeout > 0:
+                yield util.asyncSleep(.1)
+                timeout -= 1
 
+        try:
+            yield self.remoteComplete(failure)
+            # this fires the original deferred we returned from .run(),
+            self.deferred.callback(self)
+        except Exception as e:
+            self.deferred.errback(e)
+
+    @defer.inlineCallbacks
     def interrupt(self, why):
         log.msg("RemoteCommand.interrupt", self, why)
-        if not self.active or self.interrupted:
-            log.msg(" but this RemoteCommand is already inactive")
-            return defer.succeed(None)
-        if not self.conn:
-            log.msg(" but our .conn went away")
-            return defer.succeed(None)
-        if isinstance(why, Failure) and why.check(error.ConnectionLost):
-            log.msg("RemoteCommand.disconnect: lost worker")
+
+        if self.conn and isinstance(why, Failure) and why.check(error.ConnectionLost):
+            # Note that we may be in the process of interruption and waiting for the worker to
+            # return the final results when the connection is disconnected.
+            log.msg("RemoteCommand.interrupt: lost worker")
             self.conn = None
             self._finished(why)
-            return defer.succeed(None)
+            return
+        if not self.active or self.interrupted:
+            log.msg(" but this RemoteCommand is already inactive")
+            return
+        if not self.conn:
+            log.msg(" but our .conn went away")
+            return
 
         self.interrupted = True
         # tell the remote command to halt. Returns a Deferred that will fire
         # when the interrupt command has been delivered.
-        d = self.conn.remoteInterruptCommand(self.builder_name,
-                                             self.commandID, str(why))
-        # the worker may not have remote_interruptCommand
-        d.addErrback(self._interruptFailed)
-        return d
 
-    def _interruptFailed(self, why):
-        log.msg("RemoteCommand._interruptFailed", self)
-        # TODO: forcibly stop the Command now, since we can't stop it
-        # cleanly
-        return None
+        try:
+            yield self.conn.remoteInterruptCommand(self.builder_name,
+                                                   self.commandID, str(why))
+            # the worker may not have remote_interruptCommand
+        except Exception as e:
+            log.msg("RemoteCommand.interrupt failed", self, e)
 
     def remote_update(self, updates):
         """
@@ -226,36 +231,27 @@ class RemoteCommand(base.RemoteCommandImpl):
             eventually(self._finished, failure)
         return None
 
-    def _unwrap(self, log):
-        from buildbot.process import buildstep
-        if isinstance(log, buildstep.SyncLogFileWrapper):
-            return log.unwrap()
-        return log
-
     @util.deferredLocked('loglock')
-    @defer.inlineCallbacks
     def addStdout(self, data):
         if self.collectStdout:
             self.stdout += data
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            log_ = yield self._unwrap(self.logs[self.stdioLogName])
-            log_.addStdout(data)
+            self.logs[self.stdioLogName].addStdout(data)
+        return defer.succeed(None)
 
     @util.deferredLocked('loglock')
-    @defer.inlineCallbacks
     def addStderr(self, data):
         if self.collectStderr:
             self.stderr += data
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            log_ = yield self._unwrap(self.logs[self.stdioLogName])
-            log_.addStderr(data)
+            self.logs[self.stdioLogName].addStderr(data)
+        return defer.succeed(None)
 
     @util.deferredLocked('loglock')
-    @defer.inlineCallbacks
     def addHeader(self, data):
         if self.stdioLogName is not None and self.stdioLogName in self.logs:
-            log_ = yield self._unwrap(self.logs[self.stdioLogName])
-            log_.addHeader(data)
+            self.logs[self.stdioLogName].addHeader(data)
+        return defer.succeed(None)
 
     @util.deferredLocked('loglock')
     @defer.inlineCallbacks
@@ -265,15 +261,13 @@ class RemoteCommand(base.RemoteCommandImpl):
             (activateCallBack, closeWhenFinished) = self.delayedLogs[logname]
             del self.delayedLogs[logname]
             loog = yield activateCallBack(self)
-            loog = yield self._unwrap(loog)
             self.logs[logname] = loog
             self._closeWhenFinished[logname] = closeWhenFinished
 
         if logname in self.logs:
-            log_ = yield self._unwrap(self.logs[logname])
-            yield log_.addStdout(data)
+            yield self.logs[logname].addStdout(data)
         else:
-            log.msg("{}.addToLog: no such log {}".format(self, logname))
+            log.msg(f"{self}.addToLog: no such log {logname}")
 
     @metrics.countMethod('RemoteCommand.remoteUpdate()')
     @defer.inlineCallbacks
@@ -285,7 +279,7 @@ class RemoteCommand(base.RemoteCommandImpl):
 
         if self.debug:
             for k, v in update.items():
-                log.msg("Update[{}]: {}".format(k, v))
+                log.msg(f"Update[{k}]: {v}")
         if "stdout" in update:
             # 'stdout': data
             yield self.addStdout(cleanup(update['stdout']))
@@ -301,8 +295,8 @@ class RemoteCommand(base.RemoteCommandImpl):
             yield self.addToLog(logname, cleanup(data))
         if "rc" in update:
             rc = self.rc = update['rc']
-            log.msg("{} rc={}".format(self, rc))
-            yield self.addHeader("program finished with exit code %d\n" % rc)
+            log.msg(f"{self} rc={rc}")
+            yield self.addHeader(f"program finished with exit code {rc}\n")
         if "elapsed" in update:
             self._remoteElapsed = update['elapsed']
 
@@ -323,18 +317,21 @@ class RemoteCommand(base.RemoteCommandImpl):
         for name, loog in self.logs.items():
             if self._closeWhenFinished[name]:
                 if maybeFailure:
-                    loog = yield self._unwrap(loog)
-                    yield loog.addHeader("\nremoteFailed: {}".format(maybeFailure))
+                    yield loog.addHeader(f"\nremoteFailed: {maybeFailure}")
                 else:
-                    log.msg("closing log {}".format(loog))
-                loog.finish()
+                    log.msg(f"closing log {loog}")
+                yield loog.finish()
         if maybeFailure:
+            # Message Pack protocol can not send an exception object back to the master, so
+            # exception information is sent as a string
+            if isinstance(maybeFailure, str):
+                raise RemoteException(maybeFailure)
+
             # workaround http://twistedmatrix.com/trac/ticket/5507
             # CopiedFailure cannot be raised back, this make debug difficult
             if isinstance(maybeFailure, pb.CopiedFailure):
-                maybeFailure.value = RemoteException("{}: {}\n{}".format(maybeFailure.type,
-                                                                         maybeFailure.value,
-                                                                         maybeFailure.traceback))
+                maybeFailure.value = RemoteException(f"{maybeFailure.type}: {maybeFailure.value}"
+                                                     f"\n{maybeFailure.traceback}")
                 maybeFailure.type = RemoteException
             maybeFailure.raiseException()
 
@@ -421,9 +418,9 @@ class RemoteShellCommand(RemoteCommand):
                 self.args['dir'] = self.args['workdir']
             if self.step.workerVersionIsOlderThan("shell", "2.16"):
                 self.args.pop('sigtermTime', None)
-        what = "command '{}' in dir '{}'".format(self.fake_command, self.args['workdir'])
+        what = f"command '{self.fake_command}' in dir '{self.args['workdir']}'"
         log.msg(what)
         return super()._start()
 
     def __repr__(self):
-        return "<RemoteShellCommand '{}'>".format(repr(self.fake_command))
+        return f"<RemoteShellCommand '{repr(self.fake_command)}'>"
