@@ -29,6 +29,7 @@ from buildbot.process.results import SUCCESS
 from buildbot.process.results import WARNINGS
 from buildbot.reporters.base import ReporterBase
 from buildbot.reporters.generators.build import BuildStartEndStatusGenerator
+from buildbot.reporters.generators.buildrequest import BuildRequestGenerator
 from buildbot.reporters.message import MessageFormatterRenderable
 from buildbot.util import giturlparse
 from buildbot.util import httpclientservice
@@ -78,8 +79,10 @@ class GitLabStatusPush(ReporterBase):
     def _create_default_generators(self):
         start_formatter = MessageFormatterRenderable('Build started.')
         end_formatter = MessageFormatterRenderable('Build done.')
+        pending_formatter = MessageFormatterRenderable('Build pending.')
 
         return [
+            BuildRequestGenerator(formatter=pending_formatter),
             BuildStartEndStatusGenerator(start_formatter=start_formatter,
                                          end_formatter=end_formatter)
         ]
@@ -110,8 +113,7 @@ class GitLabStatusPush(ReporterBase):
         if context is not None:
             payload['name'] = context
 
-        return self._http.post('/api/v4/projects/{}/statuses/{}'.format(project_id, sha),
-                json=payload)
+        return self._http.post(f'/api/v4/projects/{project_id}/statuses/{sha}', json=payload)
 
     @defer.inlineCallbacks
     def getProjectId(self, sourcestamp):
@@ -119,18 +121,17 @@ class GitLabStatusPush(ReporterBase):
         url = giturlparse(sourcestamp['repository'])
         if url is None:
             return None
-        project_full_name = "{}/{}".format(url.owner, url.repo)
+        project_full_name = f"{url.owner}/{url.repo}"
         # gitlab needs project name to be fully url quoted to get the project id
         project_full_name = urlquote_plus(project_full_name)
 
         if project_full_name not in self.project_ids:
-            response = yield self._http.get('/api/v4/projects/{}'.format(project_full_name))
+            response = yield self._http.get(f'/api/v4/projects/{project_full_name}')
             proj = yield response.json()
             if response.code not in (200, ):
                 log.msg(
                     'Unknown (or hidden) gitlab project'
-                    '{repo}: {message}'.format(
-                        repo=project_full_name, **proj))
+                    f'{project_full_name}: {proj.get("message")}')
                 return None
             self.project_ids[project_full_name] = proj['id']
 
@@ -191,21 +192,14 @@ class GitLabStatusPush(ReporterBase):
                     message = yield res.json()
                     message = message.get('message', 'unspecified error')
                     log.msg(
-                        'Could not send status "{state}" for '
-                        '{repo} at {sha}: {message}'.format(
-                            state=state,
-                            repo=sourcestamp['repository'], sha=sha,
-                            message=message))
+                        f'Could not send status "{state}" for '
+                        f'{sourcestamp["repository"]} at {sha}: {message}')
                 elif self.verbose:
                     log.msg(
-                        'Status "{state}" sent for '
-                        '{repo} at {sha}.'.format(
-                            state=state, repo=sourcestamp['repository'], sha=sha))
+                        f'Status "{state}" sent for '
+                        f'{sourcestamp["repository"]} at {sha}.')
             except Exception as e:
                 log.err(
                     e,
-                    'Failed to send status "{state}" for '
-                    '{repo} at {sha}'.format(
-                        state=state,
-                        repo=sourcestamp['repository'], sha=sha
-                    ))
+                    (f'Failed to send status "{state}" for '
+                     f'{sourcestamp["repository"]} at {sha}'))
