@@ -97,6 +97,7 @@ class Timed(AbsoluteSourceStampsMixin, base.BaseScheduler):
                                              onlyImportant=self.onlyImportant)
         else:
             yield self.master.db.schedulers.flushChangeClassifications(self.serviceid)
+        return None
 
     @defer.inlineCallbacks
     def deactivate(self):
@@ -115,6 +116,7 @@ class Timed(AbsoluteSourceStampsMixin, base.BaseScheduler):
                 self.actuateAtTimer.cancel()
             self.actuateAtTimer = None
         yield self.actuationLock.run(stop_actuating)
+        return None
 
     # Scheduler methods
 
@@ -146,14 +148,19 @@ class Timed(AbsoluteSourceStampsMixin, base.BaseScheduler):
         scheds = self.master.db.schedulers
         classifications = yield scheds.getChangeClassifications(self.serviceid)
 
-        # if onlyIfChanged is True, then we will skip this build if no
-        # important changes have occurred since the last invocation
-        if self.onlyIfChanged and not any(classifications.values()):
-            log.msg(("%s scheduler <%s>: skipping build " +
-                     "- No important changes") %
-                    (self.__class__.__name__, self.name))
+        # if onlyIfChanged is True, then we will skip this build if no important changes have
+        # occurred since the last invocation. Note that when the scheduler has just been started
+        # there may not be any important changes yet and we should start the build for the
+        # current state of the code whatever it is.
+        last_only_if_changed = yield self.getState('last_only_if_changed', False)
+        if last_only_if_changed and self.onlyIfChanged and not any(classifications.values()):
+            log.msg(("{} scheduler <{}>: skipping build " +
+                     "- No important changes").format(self.__class__.__name__, self.name))
             return
 
+        if last_only_if_changed != self.onlyIfChanged:
+            yield self.setState('last_only_if_changed', self.onlyIfChanged)
+
         changeids = sorted(classifications.keys())
 
         if changeids:
@@ -206,6 +213,7 @@ class Timed(AbsoluteSourceStampsMixin, base.BaseScheduler):
         "Similar to util.now, but patchable by tests"
         return util.now(self._reactor)
 
+    @defer.inlineCallbacks
     def _scheduleNextBuild_locked(self):
         # clear out the existing timer
         if self.actuateAtTimer:
@@ -213,23 +221,22 @@ class Timed(AbsoluteSourceStampsMixin, base.BaseScheduler):
         self.actuateAtTimer = None
 
         # calculate the new time
-        d = self.getNextBuildTime(self.lastActuated)
+        actuateAt = yield self.getNextBuildTime(self.lastActuated)
 
-        # set up the new timer
-        @d.addCallback
-        def set_timer(actuateAt):
+        if actuateAt is None:
+            self.actuateAt = None
+        else:
+            # set up the new timer
             now = self.now()
             self.actuateAt = max(actuateAt, now)
-            if actuateAt is not None:
-                untilNext = self.actuateAt - now
-                if untilNext == 0:
-                    log.msg(("%s scheduler <%s>: missed scheduled build time"
-                             " - building immediately") %
-                            (self.__class__.__name__, self.name))
-                self.actuateAtTimer = self._reactor.callLater(untilNext,
-                                                              self._actuate)
-        return d
+            untilNext = self.actuateAt - now
+            if untilNext == 0:
+                log.msg(f"{self.__class__.__name__} scheduler <{self.name}>: "
+                        "missed scheduled build time - building immediately")
+            self.actuateAtTimer = self._reactor.callLater(untilNext,
+                                                          self._actuate)
 
+    @defer.inlineCallbacks
     def _actuate(self):
         # called from the timer when it's time to start a build
         self.actuateAtTimer = None
@@ -245,16 +252,15 @@ class Timed(AbsoluteSourceStampsMixin, base.BaseScheduler):
             self.actuateAt = None
             yield self.setState('last_build', self.lastActuated)
 
-            # start the build
-            yield self.startBuild()
-
-            # schedule the next build (noting the lock is already held)
-            yield self._scheduleNextBuild_locked()
-        d = self.actuationLock.run(set_state_and_start)
-
-        # this function can't return a deferred, so handle any failures via
-        # log.err
-        d.addErrback(log.err, 'while actuating')
+            try:
+                # start the build
+                yield self.startBuild()
+            except Exception as e:
+                log.err(e, 'while actuating')
+            finally:
+                # schedule the next build (noting the lock is already held)
+                yield self._scheduleNextBuild_locked()
+        yield self.actuationLock.run(set_state_and_start)
 
 
 class Periodic(Timed):
@@ -321,11 +327,9 @@ class NightlyBase(Timed):
 
     def getNextBuildTime(self, lastActuated):
         dateTime = lastActuated or self.now()
-        sched = '%s %s %s %s %s' % (self._timeToCron(self.minute),
-                                    self._timeToCron(self.hour),
-                                    self._timeToCron(self.dayOfMonth),
-                                    self._timeToCron(self.month),
-                                    self._timeToCron(self.dayOfWeek, True))
+        sched = (f'{self._timeToCron(self.minute)} {self._timeToCron(self.hour)} '
+                 f'{self._timeToCron(self.dayOfMonth)} {self._timeToCron(self.month)} '
+                 f'{self._timeToCron(self.dayOfWeek, True)}')
         cron = croniter.croniter(sched, dateTime)
         nextdate = cron.get_next(float)
         return defer.succeed(nextdate)
