@@ -56,12 +56,12 @@ git_describe_flags = [
     # string parameter
     ('match', lambda v: ['--match', v] if v else None),
     # numeric parameter
-    ('abbrev', lambda v: ['--abbrev={}'.format(v)]
+    ('abbrev', lambda v: [f'--abbrev={v}']
      if isTrueOrIsExactlyZero(v) else None),
-    ('candidates', lambda v: ['--candidates={}'.format(v)] if isTrueOrIsExactlyZero(v) else None),
+    ('candidates', lambda v: [f'--candidates={v}'] if isTrueOrIsExactlyZero(v) else None),
     # optional string parameter
     ('dirty', lambda v: ['--dirty'] if (v is True or v == '') else None),
-    ('dirty', lambda v: ['--dirty={}'.format(v)] if (v and v is not True) else None),
+    ('dirty', lambda v: [f'--dirty={v}'] if (v and v is not True) else None),
 ]
 
 
@@ -73,9 +73,9 @@ class Git(Source, GitStepMixin):
 
     def __init__(self, repourl=None, branch='HEAD', mode='incremental', method=None,
                  reference=None, submodules=False, remoteSubmodules=False, shallow=False,
-                 progress=True, retryFetch=False, clobberOnFailure=False, getDescription=False,
-                 config=None, origin=None, sshPrivateKey=None, sshHostKey=None, sshKnownHosts=None,
-                 **kwargs):
+                 filters=None, progress=True, retryFetch=False, clobberOnFailure=False,
+                 getDescription=False, config=None, origin=None, sshPrivateKey=None,
+                 sshHostKey=None, sshKnownHosts=None, **kwargs):
 
         if not getDescription and not isinstance(getDescription, dict):
             getDescription = False
@@ -88,6 +88,7 @@ class Git(Source, GitStepMixin):
         self.submodules = submodules
         self.remoteSubmodules = remoteSubmodules
         self.shallow = shallow
+        self.filters = filters
         self.clobberOnFailure = clobberOnFailure
         self.mode = mode
         self.prog = progress
@@ -105,8 +106,8 @@ class Git(Source, GitStepMixin):
 
         if isinstance(self.mode, str):
             if not self._hasAttrGroupMember('mode', self.mode):
-                bbconfig.error("Git: mode must be {}".format(
-                        ' or '.join(self._listAttrGroupMembers('mode'))))
+                bbconfig.error(
+                    f"Git: mode must be {' or '.join(self._listAttrGroupMembers('mode'))}")
             if isinstance(self.method, str):
                 if self.mode == 'full' and \
                         self.method not in ['clean', 'fresh', 'clobber', 'copy', None]:
@@ -266,7 +267,7 @@ class Git(Source, GitStepMixin):
         revision = stdout.strip()
         if len(revision) != GIT_HASH_LENGTH:
             raise buildstep.BuildStepFailed()
-        log.msg("Got Git revision {}".format(revision))
+        log.msg(f"Got Git revision {revision}")
         self.updateSourceProperty('got_revision', revision)
 
         return RC_SUCCESS
@@ -380,6 +381,12 @@ class Git(Source, GitStepMixin):
             command += ['--reference', self.reference]
         if self.origin:
             command += ['--origin', self.origin]
+        if self.filters:
+            if self.supportsFilters:
+                for filter in self.filters:
+                    command += ['--filter', filter]
+            else:
+                log.msg("Git versions < 2.27.0 don't support filters on clone")
         command += [self.repourl, '.']
 
         if self.prog:
@@ -401,8 +408,7 @@ class Git(Source, GitStepMixin):
         if self.retry and not done:
             delay, repeats = self.retry
             if repeats > 0:
-                log.msg("Checkout failed, trying %d more times after %d seconds"
-                        % (repeats, delay))
+                log.msg(f"Checkout failed, trying {repeats} more times after {delay} seconds")
                 self.retry = (delay, repeats - 1)
 
                 df = defer.Deferred()
@@ -523,9 +529,7 @@ class Git(Source, GitStepMixin):
             return "clone"
 
         cmd = remotecommand.RemoteCommand('listdir',
-                                          {'dir': self.workdir,
-                                           'logEnviron': self.logEnviron,
-                                           'timeout': self.timeout, })
+                                          {'dir': self.workdir})
         cmd.useLog(self.stdio_log, False)
         yield self.runCommand(cmd)
 
